<?php

namespace Drupal\depcalc\EventSubscriber\DependencyCollector;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\depcalc\DependencyCalculatorEvents;
use Drupal\depcalc\DependentEntityWrapper;
use Drupal\depcalc\Event\CalculateEntityDependenciesEvent;

class EntityLanguage extends BaseDependencyCollector {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * EntityLanguage constructor.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(ModuleHandlerInterface $module_handler) {
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[DependencyCalculatorEvents::CALCULATE_DEPENDENCIES][] = ['onCalculateDependencies'];
    return $events;
  }

  /**
   * Calculates the language of content entities.
   *
   * @param \Drupal\depcalc\Event\CalculateEntityDependenciesEvent $event
   *   The dependency calculation event.
   */
  public function onCalculateDependencies(CalculateEntityDependenciesEvent $event) {
    // @Todo figure out the content translation settings for the entity/bundle.
    $entity = $event->getEntity();
    if ($entity instanceof ContentEntityInterface && $entity instanceof TranslatableInterface) {
      /**
       * @var string $field_name
       * @var \Drupal\Core\Field\FieldItemListInterface $field
       */
      foreach ($entity as $field_name => $field) {
        if ($field->getFieldDefinition()->getType() == 'language' && $this->moduleHandler->moduleExists('language')) {
          /** @var \Drupal\language\Entity\ContentLanguageSettings $settings */
          $settings = \Drupal::entityTypeManager()->getStorage('language_content_settings')->load("{$entity->getEntityTypeId()}.{$entity->bundle()}");
          if (!$settings || !$settings->status() || !$settings->isLanguageAlterable()) {
            return;
          }
          $settings_wrapper = new DependentEntityWrapper($settings);
          $dependencies = $event->getDependencies();
          $local_dependencies = [];
          $this->mergeDependencies($settings_wrapper, $event->getStack(), $this->getCalculator()->calculateDependencies($settings_wrapper, $event->getStack(), $local_dependencies));
          $dependencies[$settings_wrapper->getUUid()] = $settings_wrapper;
          $modules = $dependencies['module'];
          $event->setModuleDependencies($dependencies['module']);
          unset($dependencies['module']);
          $event->setDependencies(...array_values($dependencies));
          $dependencies['module'] = $modules;
          foreach ($entity->getTranslationLanguages() as $language) {
            $language_entity = \Drupal::entityTypeManager()->getStorage('configurable_language')->load($language->getId());
            $language_entity_wrapper = new DependentEntityWrapper($language_entity);
            $language_entity_wrapper->addModuleDependencies(['language']);

            $local_dependencies = [];
            $this->mergeDependencies($language_entity_wrapper, $event->getStack(), $this->getCalculator()->calculateDependencies($language_entity_wrapper, $event->getStack(), $local_dependencies));
            $dependencies[$language_entity_wrapper->getEntity()->uuid()] = $language_entity_wrapper;
            $event->addDependency($language_entity_wrapper);
          }
          $event->setModuleDependencies($dependencies['module']);
          unset($dependencies['module']);
          $event->setDependencies(...array_values($dependencies));
        }
      }
    }
  }



}
