<?php

namespace Drupal\depcalc\EventSubscriber\DependencyCollector;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\depcalc\DependencyCalculatorEvents;
use Drupal\depcalc\DependentEntityWrapper;
use Drupal\depcalc\Event\CalculateEntityDependenciesEvent;

/**
 * Subscribes to dependency collection to extract referenced entities.
 */
class EntityReferenceFieldDependencyCollector extends BaseDependencyCollector {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[DependencyCalculatorEvents::CALCULATE_DEPENDENCIES][] = ['onCalculateDependencies'];
    return $events;
  }

  /**
   * Calculates the referenced entities.
   *
   * @param \Drupal\depcalc\Event\CalculateEntityDependenciesEvent $event
   *   The dependency calculation event.
   */
  public function onCalculateDependencies(CalculateEntityDependenciesEvent $event) {
    $entity = $event->getEntity();
    if ($entity instanceof ContentEntityInterface) {
      /**
       * @var string $field_name
       * @var \Drupal\Core\Field\FieldItemListInterface $field
       */
      foreach ($entity as $field_name => $field) {
        if (in_array($field->getFieldDefinition()->getType(), [
          'file',
          'image',
          'entity_reference',
          'entity_reference_revisions'
        ])) {
          $dependencies = $event->getDependencies();
          foreach ($field as $item) {
            if (!$item->entity) {
              $sub_entity = \Drupal::entityTypeManager()->getStorage($field->getFieldDefinition()->getSetting('target_type'))->load($item->getValue()['target_id']);
              if (is_null($sub_entity)) {
                continue;
              }
              $item->entity = $sub_entity;
            }
            $item_entity_wrapper = new DependentEntityWrapper($item->entity);
            $local_dependencies = [];
            $this->mergeDependencies($item_entity_wrapper, $event->getStack(), $this->getCalculator()->calculateDependencies($item_entity_wrapper, $event->getStack(), $local_dependencies));
            $dependencies[$item_entity_wrapper->getUuid()] = $item_entity_wrapper;
            $event->addDependency($item_entity_wrapper);
          }
          $event->setModuleDependencies($dependencies['module']);
          unset($dependencies['module']);
          $event->setDependencies(...array_values($dependencies));
        }
      }
    }
  }

}
